#!/bin/bash
#################################################
# 启动云协作Lite服务
# 使用说明: sh pm2Manager.sh --h
###################################
# For Ubuntu, make sure use bash instead of dash
if [ "$BASH_VERSION" = "" ]; then
	echo "请使用Bash运行此脚本，例如/bin/bash pm2Manager.sh"
	exit 1
fi

RED=$(tput setaf 1)
GREEN=$(tput setaf 2)
YELLOW=$(tput setaf 3)
NORMAL=$(tput sgr0)
STEP=1
function printError() {
	printf '%s%s\n' "$RED" "$1" "$NORMAL"
}
function printStep() {
	printf '%s%s\n' "$GREEN" "$STEP -- $1" "$NORMAL"
	((STEP++))
}
function printInfo() {
	printf '%s%s\n' "$NORMAL" "$1" "$NORMAL"
}
function printSuccess() {
	printf '%s%s\n' "$GREEN" "$1" "$NORMAL"
}
function printWarn() {
	printf '%s%s\n' "$YELLOW" "$1" "$NORMAL"
}
function exitScript() {
	exitNum=0
	if [ ! -z "${1}" ]; then
		exitNum=${1}
	fi
	if [ ! -z "${2}" ]; then
		printError "${2}"
	fi
	exit "${exitNum}"
}
function printHelp() {
	#################################################
	printInfo "此脚本用于启动云协作Lite服务（注：在Ubuntu上请使用/bin/bash运行脚本，在CentOS上使用sh运行脚本）"
	printSuccess "启动云协作Lite："
	printInfo "     sh pm2Manager.sh --start 或者 sh pm2Manager.sh --reload"
	printInfo "     同时，上述两个指令均支持指定pm2配置文件和配置文件中的环境变量模块(例如：env_production)，例如："
	printInfo "     sh pm2Manager.sh --reload --env production --config pm2_custom.config.js"
	printInfo "     如果只启动云协作Lite，可以指定 --onlyApp 例如："
	printInfo "     sh pm2Manager.sh --reload --onlyApp"
	printSuccess "停止云协作Lite："
	printInfo "     sh pm2Manager.sh --stop    停止云协作Lite，但不从pm2列表中移出"
	printInfo "     sh pm2Manager.sh --delete  停止云协作Lite，并pm2列表中移出"
	printInfo "     sh pm2Manager.sh --kill    停止PM2实例（daemon）"
	printSuccess "打印/清空PM2日志："
	printInfo "     sh pm2Manager.sh --logs  --logLines <可选参数，默认：50>  --targetApp <可选参数，PM2项目名称，例如:BaichuanApp>"
	printInfo "     sh pm2Manager.sh --flushLog 清空pm2日志"
	printSuccess "打印PM2项目信息："
	printInfo "     sh pm2Manager.sh --list"
	printInfo "     sh pm2Manager.sh --describe  --targetApp <PM2项目名称，例如:BaichuanApp>"
}
function paramValueCheck() {
	paramValue="$1"
	paramName="$2"
	if [ -z "$paramValue" ] || [[ $paramValue == --* ]]; then
		printError "no param value for $paramName"
		exit 1
	fi
}
function executeCmd() {
	CMD="$1"
	printInfo "executing $CMD"
	if ! eval "$CMD" >/dev/null 2>&1; then
		printError "命令执行失败"
		exit 1
	fi
}
# check the process existence according the key words, which use 'grep' to do it
function checkProcess() {
	# check the args
	if [ "$1" == "" ]; then
		return 0
	fi

	PROCESS_NUM=$(ps -ef | grep "$1" | grep -v "grep" | wc -l)
	if [ "$PROCESS_NUM" -ge 1 ]; then
		return 1
	else
		return 0
	fi
}
function startDB() {
	checkProcess "$ROOT_FOLDER/data/log/baichuan_db.log"
	dbRunning=$?
	if [ $dbRunning -gt 0 ]; then
		printInfo "数据库正在运行"
		return
	fi
	if [ ! -d "$ROOT_FOLDER/data" ]; then
		printWarn "未发现默认的data文件夹，无法自动启动数据库"
		return
	fi
	if [ ! -d "$ROOT_FOLDER/data/db" ]; then
		printWarn "未发现默认的data/db文件夹，无法自动启动数据库"
		return
	fi
	if [ ! -d "$ROOT_FOLDER/data/log" ]; then
		printWarn "未发现默认的data/log文件夹，无法自动启动数据库"
		return
	fi

	dbLocation=$1
	printInfo "启动数据库 ..."
	FailedToStart=0
	if [ "$dbLocation" -eq 0 ]; then
		# 数据库全局安装
		cmdStr="mongod"
	else
		# 数据库本地安装
		cmdStr="$ROOT_FOLDER/$DB_FOLDER/bin/mongod"
	fi

	cmdStr+=" --dbpath $ROOT_FOLDER/data/db --logpath $ROOT_FOLDER/data/log/baichuan_db.log --fork"
	printInfo "$cmdStr"
	if eval "$cmdStr"; then
		sleepTime=30
		while [ $sleepTime -gt 0 ]; do
			sleep 5
			checkProcess "$ROOT_FOLDER/data/log/baichuan_db.log"
			dbRunning=$?
			if [ $dbRunning -eq 1 ]; then
				printSuccess "数据库启动成功"
				break
			else
				((sleepTime -= 5))
			fi
		done
		if [ $sleepTime -le 0 ]; then
			FailedToStart=1
		fi
	else
		FailedToStart=1
	fi

	if [ $FailedToStart -eq 1 ]; then
		printError "启动数据库失败。"
		checkProcess "mongod"
		dbRunning=$?
		if [ $dbRunning -eq 1 ]; then
			printError "检测到其它mongod实例，请确认是否有端口冲突。"
		fi
		printWarn "数据库启动日志："
		logInfo=$(cat "$ROOT_FOLDER/data/log/baichuan_db.log")
		echo "$logInfo"
		exitScript 1
	fi
}

function startES() {
	checkProcess "$ROOT_FOLDER/${ES_FOLDER}/config"
	esRunning=$?
	if [ ${esRunning} -gt 0 ]; then
		printInfo "ES正在运行"
		return
	fi
	printInfo "启动ES ..."
	FailedToStart=0
	cmdStr="$ROOT_FOLDER/$ES_FOLDER/bin/elasticsearch -d"
	printInfo "$cmdStr"
	if ! eval "$cmdStr"; then
		FailedToStart=1
	else
		sleepTime=30
		while [ $sleepTime -gt 0 ]; do
			sleep 5
			checkProcess "$ROOT_FOLDER/$ES_FOLDER/config"
			esRunning=$?
			if [ $esRunning -eq 1 ]; then
				printSuccess "ES启动成功"
				break
			else
				((sleepTime -= 5))
			fi
		done
		if [ $sleepTime -le 0 ]; then
			FailedToStart=1
		fi
	fi
	if [ $FailedToStart -eq 1 ]; then
		printError "启动ES失败($cmdStr)"
		if [ -f "${ROOT_FOLDER}/${ES_FOLDER}/logs/elasticsearch.log" ]; then
			logInfo=$(cat "${ROOT_FOLDER}/${ES_FOLDER}/logs/elasticsearch.log")
			printWarn "系统启动日志："
			echo "$logInfo"
		fi
		exitScript 1
	fi
}

BAICHUAN_FOLDER=$(pwd)
cd ..
ROOT_FOLDER=$(pwd)
cd "${BAICHUAN_FOLDER}"
# script args
INST_NAME="pm2_baichuan"
ENV_NAME=""
ACTION="NONE"
KEEP_ENV=0
IS_DEBUG=0
LOG_LINES=50
TARGET_APP=""
NEED_HELP=0
ONLY_APP=0

# global variables
PM2_CONFIG_NAME="pm2.config.js"
NODE_VERSION="v8.9.4"
NODE_FOLDER="node-${NODE_VERSION}"
NODE_CMD_LOCATION="${ROOT_FOLDER}/${NODE_FOLDER}/bin"

DB_VERSION="3.6.4"
DB_FOLDER="mongodb-${DB_VERSION}"
DB_CMD_LOCATION="${ROOT_FOLDER}/${DB_FOLDER}/bin/mongod"

ES_VERSION="6.2.2"
ES_FOLDER="elasticsearch-${ES_VERSION}"
ES_CMD_LOCATION="${ROOT_FOLDER}/${ES_FOLDER}/bin/elasticsearch"

CMD=""
OTHER_PM2_ARGS=""
IS_PM2_GLOBAL=0

# 解析脚本支持的参数
while true; do
	case "$1" in
		--instName)
			paramValueCheck "$2" "$1"
			INST_NAME="$2"
			shift 2
			;;
		--env)
			paramValueCheck "$2" "$1"
			ENV_NAME="$2"
			shift 2
			;;
		--start)
			ACTION="start"
			shift 1
			;;
		--stop)
			ACTION="stop"
			shift 1
			;;
		--reload)
			ACTION="reload"
			shift 1
			;;
		--restart)
			ACTION="restart"
			shift 1
			;;
		--list)
			ACTION="list"
			shift 1
			;;
		--kill)
			ACTION="kill"
			shift 1
			;;
		--logs)
			ACTION="logs"
			shift 1
			;;
		--logLines)
			paramValueCheck "$2" "$1"
			LOG_LINES="$2"
			shift 2
			;;
		--targetApp)
			paramValueCheck "$2" "$1"
			TARGET_APP="$2"
			shift 2
			;;
		--flushLog)
			ACTION="flush"
			shift 1
			;;
		--describe)
			ACTION="describe"
			shift 1
			;;
		--debug)
			IS_DEBUG=1
			shift 1
			;;
		--keep-env | --keepEnv)
			KEEP_ENV=1
			shift 1
			;;
		--config)
			paramValueCheck "$2" "$1"
			PM2_CONFIG_NAME="$2"
			shift 2
			;;
		--onlyApp)
			ONLY_APP=1
			shift 1
			;;
		--h)
			NEED_HELP=1
			shift 1
			;;
		*)
			if [ -z "$1" ]; then
				break
			else
				printWarn "unsupported param:$1"
				exit 1
			fi
			;;
	esac
done
if [ ${IS_DEBUG} -eq 1 ]; then
	printInfo "RootFolder:${ROOT_FOLDER}  BaiChuanFolder:${BAICHUAN_FOLDER}"
fi
if [ ${NEED_HELP} -eq 1 ]; then
	printHelp
	exit 0
fi

if [ ${ACTION} == "NONE" ]; then
	printError "请指定要执行的操作，例如：--reload, --kill等等"
	exit 1
fi
printStep "检测运行环境，包括配置文件，pm2依赖等"
if [ ! -f "./${PM2_CONFIG_NAME}" ]; then
	printError "未在当前目录发现PM2配置文件：$PM2_CONFIG_NAME"
	exit 1
fi
if [[ ! ${PM2_CONFIG_NAME} =~ ^.+\.config.js$ ]]; then
	printError "配置文件名称格式有误，请遵循此格式：<any name>.config.js"
	exit 1
fi
# 检测是否有node命令，如果没有，则查看是否在上级目录安装有nodejs，如果有，则临时设置PATH
if ! command -v node >/dev/null 2>&1; then
	if [ -d "$NODE_CMD_LOCATION" ]; then
		export PATH=$PATH:$NODE_CMD_LOCATION
		printWarn "在PATH环境变量中，设置Node命令：$NODE_CMD_LOCATION"
	else
		printError "未发现node命令，无法启动云协作Lite。请设置Path环境变量，指向nodejs的bin目录。"
		exit 1
	fi
fi

if ! command -v pm2 >/dev/null 2>&1; then
	if [ ! -f "./node_modules/pm2/bin/pm2" ]; then
		printError "未发现全局或者本地的pm2应用，无法启动云协作Lite"
		exit 1
	fi
else
	IS_PM2_GLOBAL=1
fi
if [ "${ONLY_APP}" -eq 0 ] && ([ "$ACTION" == "start" ] || [ "$ACTION" == "reload" ]); then
	# 检测是否本机安装了Mongodb，如果安装了但没有启动，则启动
	isDBReady=0
	if [ -f "${DB_CMD_LOCATION}" ]; then
		startDB 1
		isDBReady=1
	elif command -v mongod >/dev/null 2>&1; then
		rawVersion=$(mongod --version)
		echo "$rawVersion" | grep v3.6 && isDBReady=1
		if [ ${isDBReady} -eq 1 ]; then
			startDB 0
			isDBReady=1
		fi
	fi
	if [ ${isDBReady} -eq 0 ]; then
		printWarn "未在本机发现有效的mongodb安装"
	fi

	# 检测是否本机安装了ES，如果安装了但没有启动，则启动
	if [ -f "${ES_CMD_LOCATION}" ]; then
		startES
	else
		printWarn "未在本机发现有效的ES安装"
	fi
fi
if [ $IS_DEBUG -eq 1 ]; then
	CMD+="DEBUG=*"
fi
if [ "$ACTION" == "kill" ]; then
	# for kill, don't need any param
	PM2_CONFIG_NAME=""
	OTHER_PM2_ARGS=""
elif [ "$ACTION" == "logs" ]; then
	PM2_CONFIG_NAME=""
	OTHER_PM2_ARGS="--nostream --lines $LOG_LINES"
	if [ "$TARGET_APP" != "" ]; then
		OTHER_PM2_ARGS+=" /$TARGET_APP/"
	fi
elif [ "$ACTION" == "flush" ]; then
	PM2_CONFIG_NAME=""
	OTHER_PM2_ARGS=""
elif [ "$ACTION" == "describe" ]; then
	if [ "$TARGET_APP" == "" ]; then
		printError "请指定参数--targetApp，目前支持的值：BaichuanApp,BaichuanSearch"
		exit 1
	fi

	PM2_CONFIG_NAME=""
	OTHER_PM2_ARGS="$TARGET_APP"
elif [ "$ACTION" == "stop" ]; then
	if [ "$TARGET_APP" != "" ]; then
		PM2_CONFIG_NAME=""
		OTHER_PM2_ARGS="$TARGET_APP"
	fi
fi

if [ $IS_PM2_GLOBAL -eq 0 ]; then
	EXEC_CMD="./node_modules/pm2/bin/pm2"
else
	EXEC_CMD="pm2"
fi
CMD+=" PM2_HOME=$INST_NAME $EXEC_CMD $ACTION $PM2_CONFIG_NAME $OTHER_PM2_ARGS"
if [ "$ENV_NAME" != "" ]; then
	CMD+=" --env $ENV_NAME"
fi
if [ $KEEP_ENV -eq 0 ]; then
	CMD+=" --update-env"
fi
printStep "运行命令: $CMD"
eval "$CMD"
